/* Sample program for precept 9, where the topics are Palm databases
 * and debugging. Hence there are bugs in this program.
 */

#include <PalmOS.h>
#include "resource.h"
#include "ele101.h"

#define CREATOR 'pj7u'

void save_data(void);
void load_data(void);

DmOpenRef p_mydb;

Boolean MainFormHandleEvent(EventPtr evt) {
    Boolean handled = false;
    FormPtr currentform;

    switch (evt->eType) {
        case frmOpenEvent:
            p_mydb = DmOpenDatabaseByTypeCreator(DBTYPE, CREATOR, dmModeReadWrite);
            if (!p_mydb) {
                Err db_err;
                db_err = DmCreateDatabase(CARDNO, "myrecord", CREATOR, DBTYPE, false);
                ErrFatalDisplayIf(db_err, "Could not create database");
                p_mydb = DmOpenDatabaseByTypeCreator(DBTYPE, CREATOR, dmModeReadWrite);
                ErrFatalDisplayIf(!p_mydb, "Created but couldn't find database");
            }
            currentform = FrmGetActiveForm();
            FrmDrawForm(currentform);
            break;
        case frmCloseEvent:
            DmCloseDatabase(p_mydb);
            handled = true;
            break;
        case ctlSelectEvent:
            switch (evt->data.ctlEnter.controlID) {
                case savebutton:
                    save_data();
                    handled = true;
                    break;
                case retrievebutton:
                    load_data();
                    handled = true;
                    break;
            }
            break;
        default:
            break;
    }
    return handled;
}

void save_data(void) {
    /* Save the contents of the text field into our data base. */
    char *str;
    FieldPtr field;
    MemHandle h_newrecord;
    void* p_newrecord;
    Int16 newindex = 0;

    /* If this were a sorted database, we could use DmFindSortPosition()
     * to choose our insertion point, but instead we'll simply always
     * insert a new entry into the first position. */

    h_newrecord = DmNewRecord(p_mydb, &newindex, MYMAXCHARS);
    /* Something to ponder: if the record had already existed, could
     * we have used DmGetRecord() instead of DmNewRecord()? How would
     * this change our program's behavior? */
    ErrFatalDisplayIf(!h_newrecord, "Couldn't create new record");
    p_newrecord = MemHandleLock(h_newrecord);
    field = FrmGetObjectPtr(FrmGetActiveForm(),
              FrmGetObjectIndex(FrmGetActiveForm(), namefield));
    str = FldGetTextPtr(field);
    if (DmWrite(p_newrecord, 0, str, MYMAXCHARS) != errNone)
        ErrDisplay("Unable to write record");
    MemHandleUnlock(h_newrecord);
    DmReleaseRecord(p_mydb, 0, false);
}

void load_data(void) {
    /* Retrieve the saved string from out database and print it directly
     * to the right of the 'Retrieve' button. */
    MemHandle h_newrecord;
    Char *p_newrecord;

    h_newrecord = DmQueryRecord(p_mydb, 0);
    if (h_newrecord) {
        p_newrecord = MemHandleLock(h_newrecord);
        WinDrawChars("                                ", 30, 105, 60);
        WinDrawChars(p_newrecord, StrLen(p_newrecord), 105, 60);
        MemHandleUnlock(h_newrecord);
    }
    else
        ErrDisplay("Record #0 not found");
}

UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    if(cmd == sysAppLaunchCmdNormalLaunch) {
        UInt16 errorcode;
        errorcode = StartApplication();
        if (errorcode != 0)
            return errorcode;
        EventLoop();
        StopApplication();
        return 0;
    } 
    else
        return sysErrParamErr;
}
